package liviu.tudor.convertor;

import java.io.IOException;

import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.Image;
import javax.microedition.lcdui.List;

import liviu.tudor.convertor.provider.AngleConvertor;
import liviu.tudor.convertor.provider.AreaConvertor;
import liviu.tudor.convertor.provider.AstronomicConvertor;
import liviu.tudor.convertor.provider.ConvertProvider;
import liviu.tudor.convertor.provider.DistanceConvertor;
import liviu.tudor.convertor.provider.EnergyConvertor;
import liviu.tudor.convertor.provider.FrequencyConvertor;
import liviu.tudor.convertor.provider.PressureConvertor;
import liviu.tudor.convertor.provider.SpeedConvertor;
import liviu.tudor.convertor.provider.TemperatureConvertor;
import liviu.tudor.convertor.provider.VolumeConvertor;
import liviu.tudor.convertor.provider.WeightConvertor;

/**
 * Our main midlet screen.
 * 
 * @author Liviu Tudor
 */
public class ConvertorDisplayable extends List implements CommandListener
{
    /**
     * These are all the providers "known" to the application.
     * If you write your own convertor do not forget to add it to this array.
     */
    public static final ConvertProvider PROVIDERS[] = new ConvertProvider[] {
                                                        new DistanceConvertor(),
                                                        new AreaConvertor(),
                                                        new VolumeConvertor(),
                                                        new WeightConvertor(),
                                                        new SpeedConvertor(),
                                                        new TemperatureConvertor(),
                                                        new AngleConvertor(),
                                                        new AstronomicConvertor(),
                                                        new PressureConvertor(),
                                                        new EnergyConvertor(),
                                                        new FrequencyConvertor()
                                                      };
    /**
     * Default image to be shown for a provider that doesn't have an image
     */
    private Image defaultImage;
    
    /**
     * Command fired when selecting a convertor
     */
    private Command cmdSelect = new Command( "Select", Command.ITEM, 1 );
    
    /**
     * Command fired when selecting "Info" about a convertor
     */
    private Command cmdInfo = new Command( "Info", Command.ITEM, 2 );
    
    /**
     * Form that will be dealing with user input and conversion
     */
    private ProviderInput inputDlg = new ProviderInput();
    /**
     * Constructor
     */
    public ConvertorDisplayable()
    {
        super( "Convertor App by Liv", IMPLICIT );
        
        /*
         * Create default objects
         */
        try 
        {
            defaultImage = Image.createImage( "/img/convertor.png" );
        } 
        catch (IOException e) 
        {
            defaultImage = null;
        }
        
        
        /*
         * Create a menu with all the providers
         */
        scanProviders();
        
        /*
         * Add commands
         */
        addCommand( new Command("Exit", Command.EXIT, 10) );
        addCommand( cmdSelect );
        addCommand( cmdInfo );
        setSelectCommand( cmdSelect );
        
        /*
         * Set the listener
         */
        setCommandListener( this );
    }
    
    /**
     * Scans the {@link #PROVIDERS PROVIDERS} array and builds up a visual menu with all the providers.
     * The menu will display the provider name and the provider image.
     * 
     * @see ConvertProvider#getName()
     * @see ConvertProvider#getImage()
     */
    protected void scanProviders()
    {
        deleteAll();
        for( int i = 0; i < PROVIDERS.length; i++ )
        {
            Image img = PROVIDERS[i].getImage();
            if( img == null ) img = defaultImage;
            append( PROVIDERS[i].getName(), img );
        }
    }

    /**
     * Cleanup function, typically called before exit.
     */
    protected void cleanup()
    {
        setCommandListener( null );
    }
    
    protected ConvertProvider getSelectedProvider()
    {
        if( getSelectedIndex() == -1 )
            return null;
        return PROVIDERS[ getSelectedIndex() ];
    }

    /**
     * Called when the user presses a command
     */
    public void commandAction( Command c, Displayable d ) 
    {
        if( d != this )
            return;
        
        /*
         * Exit
         */
        if( (c.getCommandType() == Command.EXIT) || 
            (c.getCommandType() == Command.CANCEL) || 
            (c.getCommandType() == Command.STOP) )
        {
            ConvertorMidlet.quitApp();
            return;
        }
        
        ConvertProvider currProvider = getSelectedProvider();
        /*
         * OK or Select
         */
        if( (c.getCommandType() == Command.OK) || (c == cmdSelect) )
        {
            if( currProvider == null )
                return;
            inputDlg.setProvider( currProvider );
            ConvertorMidlet.openScreen( inputDlg );
        }
        
        /*
         * Provide info
         */
        if( c == cmdInfo )
        {
            if( currProvider == null )
                return;
            
            ConvertorMidlet.showMessage( currProvider.getName(), currProvider.getInfo() );
        }
    }
}
